import uuid

from typing import TYPE_CHECKING

from sqlalchemy import Boolean, ForeignKey, Integer, UniqueConstraint, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .regime import Regime
    from .voucher import Voucher


@reg.mapped_as_dataclass(unsafe_hash=True)
class Bill:
    __tablename__ = "bills"
    __table_args__ = (
        UniqueConstraint("voucher_id", "regime_id"),
        UniqueConstraint("regime_id", "bill_number"),
    )

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    regime_id: Mapped[int] = mapped_column("regime_id", Integer, ForeignKey("regimes.id"), nullable=False)
    voucher_id: Mapped[uuid.UUID] = mapped_column(
        "voucher_id", UUID(as_uuid=True), ForeignKey("vouchers.id"), nullable=False
    )
    is_valid: Mapped[bool] = mapped_column("is_valid", Boolean, nullable=False)
    bill_number: Mapped[int] = mapped_column("bill_number", Integer)

    voucher: Mapped["Voucher"] = relationship(back_populates="bills")
    regime: Mapped["Regime"] = relationship(back_populates="bills")

    def __init__(self, regime_id, is_valid, bill_number):
        self.regime_id = regime_id
        self.is_valid = is_valid
        self.bill_number = bill_number
