import uuid

from typing import TYPE_CHECKING, Optional

from sqlalchemy import Boolean, ForeignKey, Integer, Unicode, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .overview import Overview
    from .section import Section


@reg.mapped_as_dataclass(unsafe_hash=True)
class FoodTable:
    __tablename__ = "food_tables"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    name: Mapped[str] = mapped_column("name", Unicode(255), nullable=False, unique=True)
    seats: Mapped[int] = mapped_column("seats", Integer, nullable=False)
    section_id: Mapped[uuid.UUID] = mapped_column(
        "section_id", UUID(as_uuid=True), ForeignKey("sections.id"), nullable=False
    )
    is_active: Mapped[bool] = mapped_column("is_active", Boolean, nullable=False)
    sort_order: Mapped[bool] = mapped_column("sort_order", Integer, nullable=False)

    section: Mapped["Section"] = relationship("Section", foreign_keys=section_id)
    status: Mapped[Optional["Overview"]] = relationship(back_populates="food_table", uselist=False)

    @property
    def __name__(self):
        return self.name

    def __init__(
        self,
        name=None,
        seats=None,
        section_id=None,
        is_active=None,
        sort_order=0,
        id_=None,
    ):
        self.name = name
        self.seats = seats
        self.section_id = section_id
        self.is_active = is_active
        self.sort_order = sort_order
        self.id = id_
