import uuid

from typing import TYPE_CHECKING, List

from sqlalchemy import Boolean, Integer, Unicode, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .product_version import ProductVersion


@reg.mapped_as_dataclass(unsafe_hash=True)
class MenuCategory:
    __tablename__ = "menu_categories"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    name: Mapped[str] = mapped_column("name", Unicode(255), nullable=False, unique=True)

    is_active: Mapped[bool] = mapped_column("is_active", Boolean, nullable=False)
    is_fixture: Mapped[bool] = mapped_column("is_fixture", Boolean, nullable=False)
    sort_order: Mapped[int] = mapped_column("sort_order", Integer, nullable=False)

    products: Mapped[List["ProductVersion"]] = relationship(back_populates="menu_category")

    def __init__(self, name, is_active, sort_order, is_fixture=False, id_=None):
        self.name = name
        self.is_active = is_active
        self.sort_order = sort_order
        self.is_fixture = is_fixture
        self.id = id_
