import uuid

from typing import TYPE_CHECKING, List

from sqlalchemy import Boolean, Unicode, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .device import Device


@reg.mapped_as_dataclass(unsafe_hash=True)
class Section:
    __tablename__ = "sections"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    name: Mapped[str] = mapped_column("name", Unicode(255), unique=True, nullable=False)
    is_fixture: Mapped[bool] = mapped_column("is_fixture", Boolean, nullable=False, insert_default=False)

    devices: Mapped[List["Device"]] = relationship(back_populates="section")

    def __init__(self, name=None, is_fixture=None, id_=None):
        self.id = id_
        self.name = name
        self.is_fixture = is_fixture

    @classmethod
    def main(cls):
        return uuid.UUID("3f13f6e7-dc76-4fca-8fdb-b2bbf29b35df")
